function varargout = point_values(H,dx,algorithm)

% POINT_VALUES -  compute multiwavelet point values
%
%        [xphi,phi,xpsi1,psi1,...] = point_values(H,dx,algorithm)
%
% Compute the point values of the scaling function and wavelet(s)
% at all points of the form n*dx which fall inside the support of phi.
% dx must be of the form 1/m^n, m = dilation factor. Default is dx = 1.
%
% Algorithm is either 'eigenvalue' (the default) or 'cascade'. The
% eigenvalue approach gives precise values. However, if phi has a singularity 
% at an integer value, the eigenvalue approach will fail. 
% The cascade algorithm will always work, but it will only give
% approximate values, and the values at a fixed point may change
% from iteration to iteration.
%
% The execution time for both algorithms is about the same (and
% quite long for multiwavelets).

% Copyright (c) 2004 by Fritz Keinert (keinert@iastate.edu),
% Dept. of Mathematics, Iowa State University, Ames, IA 50011.
% This software may be freely used and distributed for non-commercial
% purposes, provided this copyright statement is preserved, and
% appropriate credit for its use is given.
%
% Last update: Feb 20, 2004

% default arguments
if (nargin < 2)
    dx = 1;
end

if (nargin < 3)
    algorithm = 'eigenvalue';
end

H = symbol(H);
r = H.r;
m = H.m;

algorithm = lower(algorithm(1));
switch algorithm

 case 'e'
  
% build refinement matrix, find values of phi at integers
  T = refinement_matrix(trim(H(1:r,:)));
  phi = nullspace(T-eye(size(T)));
  if (size(phi,2) == 0)
      disp('eigenvalues of T are');
      eig(T);
      error('no eigenvector found');
  end
  phi = phi(:,1);
  phi = reshape(phi,r,length(phi)/r);
  a = ceil(H.min/(m-1));
  b = floor(H.max/(m-1));
  
 case 'c'
  
  % set initial values to be hat function inside support
  a = ceil(H.min/(m-1));
  b = a + 2;
  phi = zeros(r,b-a+1);
  phi(1,2) =1;

 otherwise
  
  error('unrecognized algorithm')
end

n_refine = round(-log(dx)/log(m));

if (n_refine == 0 & nargout > 2)
    disp('finding wavelet function values requires at least one subdivision');
    disp('using stepsize');
    dx = 1/m
    n_refine = 1;
end

% find the correct normalization

[p,Y] = approximation_order(H);
mu0 = Y(:,1);
mu0 = mu0 / sqrt(mu0'*mu0);
c = mu0' * (sum(phi')');
phi = phi / c;

% refine to get values at points of the form n*dx

% put the factor of sqrt(m) into H
H = H * m;

for i = 1:n_refine-1
    [phi,a,b] = refine(H(1:r,:),phi,a,b,i);
end

phinm1 = phi;

for k = 1:min(m,nargout/2)
    if (n_refine > 0)
	[phi,a,b] = refine(H((k-1)*r+1:k*r,:),phinm1,a,b,n_refine);
    end
    xphi = a:dx:b;
%    if (k == 1)
% find normalization constant
% 	rho = max(abs(eig(double(T))));
% 	n = round(log(rho)/log(2));
% 	c = zeros(r,1);
% 	for i = 1:r
% 	    c(i) = (phi(i,1:1/dx:end) * ((H(1:r,:).min:H(1:r,:).max)').^n) * (-1)^n / factorial(n);
% 	end
% 	[m,i] = max(abs(c));
% 	c = c(i);
% 	[c,i] = max(abs(phin(:)));
% 	c = phin(i);
%    end
    varargout{2*k-1} = xphi;
    varargout{2*k} = phi;
end




function [phinew,anew,bnew] = refine(H,phi,a,b,level)

% one refinement step for phi on [a,b]

r = H.r;
m = H.m;
n = H.length;
nphi = size(phi,2);
step = m^(level-1);
phinew = zeros(r,nphi+(n-1)*step);
if isa(H.coef,'sym')
    phinew = sym(phinew);
end
for k =1:n
    phinew(:,(k-1)*step+1:(k-1)*step+nphi) = ...
	phinew(:,(k-1)*step+1:(k-1)*step+nphi) + H.coef(:,:,k) * phi;
end
anew = (a + H.min)/m;
bnew = (b + H.max)/m;
